;CGeFT.pro
;
;CME Geo-effectiveness Forecast Tool (CGeFT)
;based on the empirical statistical model for predicting geomagnetic storm levels
;
;INPUT FILES:
;
;cgeft_input.txt
;=6x1 array of parameters:
;column 1: v - speed of CME in km/s, float, range v ≥ 400 km/s
;column 2: r - source position distance from the center of the solar disc in Rsun, float, range 0 < r ≤ 1
;column 3: w - width of a CME, integer, possible values: 1 (non-halo), 2(partial halo) or 3 (halo)
;column 4: f - flare class, integer, possible values: 1(B or C flare), 2 (M flare) or 3 (X flare)
;column 5: i - interaction level, integer, possible values: 1(none), 2 (low), 3 (medium), 4 (high)
;column 6: file_name - file name for the output files, string
;If a value for one or more of the parameters 1-5 is given which is outside of the possible range of values, the calculation proceeds without this/these variable(s).
;At least one of the input parameters (1-5) has to be within the possible range of values in order to proceed with the calculation, otherwise the warning is sent out.
;
;parameters.txt
;=5x7 array of distribution parameters:
;columns 1-3: parameters for the linear, power law or quadratic function which calculates the mean of the distribution
;columns 4-7: parameters to be added/subtracted from the geometric distribution to obtain adjusted distribution
;
;thresholds.txt
;=3x2 array of threshold parameters:
;column 1: tresholds for 1st bin probability, Fr(k=1) (probability for k=1, i.e. |Dst| < 100 nT)
;column 2: tresholds for 3rd bin probability, Fr(k=3) (probability for k=3, i.e. 200 < |Dst| < 300 nT)
;column 3: tresholds for 4th bin probability, Fr(k=4) (probability for k=4, i.e. |Dst| > 300 nT)
;row 1: thresholds that determine whether |Dst| < 100 nT
;row 2: thresholds that determine whether |Dst| > 200 nT
;
;
;OUTPUT FILES:
;
;cgeft_distributions.txt
;= 6x4 array of distributions:
;column 1: probability based on speed
;column 2: probability based on source
;column 3: probability based on width
;column 4: probability based on flare
;column 5: probability based on interaction
;column 6: combined probability
;Rows represent separate Dst bins:
;row 1: less than 100nT
;row 2: 100-200nT
;row 3: 200-300 nT
;row 4: more than 300 nT
;
;cgeft_table_text.txt
;= 4x6 array of text for each of the coulmn of cgeft_distributions.txt
;
;cgeft_text.txt
;= 1x2 array of online text:
;row 1: expected |Dst| level
;row 2: corresponding image name
;
;cgeft.png
;= image
;
;*****************************************************************************************************************************************************************
;
;CALLS UPPON:
;gdm_probability.pro
;dst_prediction.pro
;aarow.pro
;
;*****************************************************************************************************************************************************************
;

PRO cgeft

;define file path
path=file_dirname(routine_filepath("cgeft"))
pref_set, "IDL_PATH", path + ";<IDL_DEFAULT>", /COMMIT
resolve_all

;read-in input (solar) data from file
openr,lun, path + '\INPUT\cgeft_input.txt',/get_lun
solar_data=make_array(5,1,/float)
readf, lun, solar_data
free_lun,lun


;conditions
conditions=make_array(11,1)
condition=(solar_data(0) LT 400) or (solar_data(0) GT 5000)
conditions(0)=(solar_data(1) LE 0) or (solar_data(1) GT 1)
for i=1,3,1 do begin
conditions(0+i)=solar_data(2) NE i
endfor
for i=1,3,1 do begin
conditions(3+i)=solar_data(3) NE i
endfor
for i=1,4,1 do begin
conditions(6+i)=solar_data(4) NE i
endfor
for i=0,10,1 do begin
condition=condition and conditions(i)
endfor
if (condition) then begin
output_text="Input data are missing. Prediction not possible."
openw,lun,path + '\OUTPUT\cgeft_text.txt',/get_lun
printf,lun,output_text
free_lun,lun
endif else begin
    
;read-in parameters from file
openr,lun,path + '\MODEL_DATA\cgeft_parameters.txt',/get_lun
parameters=make_array(5,7,/float)
readf,lun,parameters
free_lun,lun

;read-in thresholds from file
openr,lun,path + '\MODEL_DATA\cgeft_thresholds.txt',/get_lun
thresholds=make_array(3,2,/float)
readf,lun,thresholds
free_lun,lun

;make distributions (call cgeftgdm_probability.pro)
cgeftgdm_probability, solar_data, parameters, distributions, output_text

;make predictions (call dst_prediction.pro)
p_combined=make_array(1,4,/float)
p_combined=distributions(5,0:3)
dst_prediction, p_combined, thresholds, Dst_expected

;plot predictions using Z-buffer:
thisDevice = !D.Name
Set_Plot, 'Z'
Erase
Device, Set_Resolution=[750,500],Set_Pixel_Depth=24, Decomposed=1

;find decomposed color:
;clr=[R,G,B], R,G,B=0,255
;somecolor = clr[0] + clr[1]*2L^8 + clr[2]*2L^16
;print, somecolor

labels=['<100','100-200','200-300','>300']
c=11229501;cobalt;13107200;55295;gold;0;black;4605510;charcoal;13107200;blue;200 red
colors=[c,c,c,c]
c1=16777215;13499135;lemonchiffon;7346457;midnight blue;;12701133;ivory gray;4605510;charcoal;7237230;dark gray;13816530;medium gray;15132390;light gray;16777215;white
c2=0;55295;gold;7346457;midnight blue;16777215;16777215;white;0;black
c3=51200;green
c4=55295;gold
c5=42495;orange
c6=200;red
xtitle='!3|Dst| (nT)'
ytitle='!3Probability'
title='Probability distribution and expected |Dst| level'
figure_name=path + '\OUTPUT\cgeft_image.png'

;text_box:
text_box=make_array(1,6, /string)

text_box(0)='INPUT:'

if ((solar_data(0) GE 400) and (solar_data(0) LE 5000)) then begin
text_box(1)='v = '+ strtrim(string(solar_data(0),format='(f10.2)'),2)+' km/s'
endif else begin
text_box(1)='v = missing'
endelse

if ((solar_data(1) GT 0) and (solar_data(1) LE 1)) then begin
text_box(2)='r = '+strtrim(string(solar_data(1),format='(f10.2)'),2)+' Rsun'
endif else begin
text_box(2)='r = missing'
endelse

case solar_data(2) of
1: text_box(3)='w = non-halo'
2: text_box(3)='w = partial halo'
3: text_box(3)='w = halo'
else: text_box(3)='w = missing'
endcase

case solar_data(3) of
1: text_box(4)='f = B or C-class flare'
2: text_box(4)='f = M-class flare'
3: text_box(4)='f = X-class flare'
else: text_box(4)='f = missing'
endcase

case solar_data(4) of
1: text_box(5)='i = no interaction'
2: text_box(5)='i = interaction not likely'
3: text_box(5)='i = probable interaction'
4: text_box(5)='i = interaction highly probable'
else: text_box(5)='i = missing'
endcase

xl=2
xloc=[xl,xl,xl,xl,xl,xl]
yloc=[0.9,0.83,0.76,0.69,0.62,0.55]

;Figures:
case Dst_expected of
'1': begin
      output_text(6,0)='|Dst| < 100 nT'
          
      !Y.RANGE = [0,1]
      !X.RANGE = [0,4]
      !P.CHARSIZE = 1.5
      !P.CHARTHICK = 1
      !P.FONT = 2
      !X.MARGIN = [8,2]
      !Y.MARGIN = [4,3]
      !P.COLOR = c2
      Bar_Plot, p_combined, barnames=labels, barspace=0.02, colors=colors, title=title, xtitle=xtitle, ytitle=ytitle, /outline, background=c1
      aarrow, 0.5, 0.87, 0.5, 0.72, /Data, fill=c3, length=5, width=20, shaft=10, color=c3
      XYOuts, xloc, yloc, text_box, /data, charsize=1.3, charthick=1, color=c2
      image = tvrd(true=1)
      write_png, figure_name, image
     end
    
'2': begin
      output_text(6,0)='100 nT < |Dst| < 200 nT'

      !Y.RANGE = [0,1]
      !X.RANGE = [0,4]
      !P.CHARSIZE = 1.5
      !P.CHARTHICK = 1
      !P.FONT = 2
      !X.MARGIN = [8,2]
      !Y.MARGIN = [4,3]
      !P.COLOR = c2
      Bar_Plot, p_combined, barnames=labels, barspace=0.02, colors=colors, title=title, xtitle=xtitle, ytitle=ytitle, /outline, background=c1
      aarrow, 1.5, 0.55, 1.5, 0.40, /Data, fill=c5, length=5, width=20, shaft=10, color=c5
      XYOuts, xloc, yloc, text_box, /data, charsize=1.3, charthick=1, color=c2
      image = tvrd(true=1)
      write_png, figure_name, image
     end
    
'3': begin
      output_text(6,0)='|Dst| > 200 nT'

      !Y.RANGE = [0,1]
      !X.RANGE = [0,4]
      !P.CHARSIZE = 1.5
      !P.CHARTHICK = 1
      !P.FONT = 2
      !X.MARGIN = [8,2]
      !Y.MARGIN = [4,3]
      !P.COLOR = c2
      Bar_Plot, p_combined, barnames=labels, barspace=0.02, colors=colors, title=title, xtitle=xtitle, ytitle=ytitle, /outline, background=c1
      aarrow, 3, 0.45, 3, 0.30, /Data, fill=c6, length=5, width=30, shaft=20, color=c6
      XYOuts, xloc, yloc, text_box, /data, charsize=1.3, charthick=1, color=c2
      image = tvrd(true=1)
      write_png, figure_name, image
     end
endcase

Set_Plot, thisDevice  

out_array=strarr(1,4)
out_array(0)='|Dst| < 100 nT'
out_array(1)='100 nT < |Dst| < 200 nT'
out_array(2)='200 nT < |Dst| < 300 nT'
out_array(3)='|Dst| > 300 nT'
;str={text:strarr(1,4),data:fltarr(6,4)}
;str.text=out_array
;str.data=distributions


;save distributions in a text file:
openw,lun,path + '\OUTPUT\cgeft_distributions.txt',/get_lun
printf,lun,format='("|Dst|_level",5x,"P_v",5x,"P_r",5x,"P_w",5x,"P_f",5x,"P_i",5x,"P_combined")'
printf,lun,distributions(0:5,0),format='("|Dst|<100nT",1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4)'
printf,lun,distributions(0:5,1),format='("100nT<|Dst|<200nT",1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4)'
printf,lun,distributions(0:5,2),format='("200nT<|Dst|<300nT",1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4)'
printf,lun,distributions(0:5,3),format='("|Dst|>300nT",1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4)'
free_lun,lun

table_text=make_array(6,1, /string)
table_text=output_text(0:5,0)
online_text=make_array(1,4, /string)
online_text(0,0)='Based on the probability distribution for a given CME (blue histogram), the expected |Dst| level calculated by the model is '$
 + output_text(6,0) + ' (marked by arrow).'
online_text(0,1)=output_text(5,0) + '(see table and table description). '
online_text(0,2)='For model calculation details see documentation.'
online_text(0,3)=figure_name

;save table_text in a text file:
openw,lun,path + '\OUTPUT\cgeft_table_text.txt',/get_lun
printf,lun,table_text
free_lun,lun

;save online_text in a text file:
openw,lun,path + '\OUTPUT\cgeft_text.txt',/get_lun
printf,lun,online_text
free_lun,lun

endelse

end