;cgeftgdm_probability.pro
;
;procedure for generating Dst probability distribution, i.e. calculating probability of a Dst level.
;The distribution is based on the discrete shifted geometric distribution.
;
;The procedure needs two arrays as an input: input_data and parameters.
;
;INPUT_DATA
;5x1 array of input variables:
;column 1: speed of CME in km/s, continuous variable, limitation: v ≥ 400 km/s
;column 2: source position radius in Rsun, continuous variable, limitations: 0 < R ≤ 1
;column 3: width of a CME, discrete variable, obtains values: 1 (non-halo), 2(partial halo) or 3 (halo)
;column 4: flare class, discrete variable, obtains values: 1(B or C flare), 2 (M flare) or 3 (X flare)
;column 5: interaction level, discrete variable, obtains values: 1(none), 2 (low), 3 (medium), 4 (high)
;If a parameter is unknown, a value 10 should be given instead (outside limitations of all variables).
;If a value for the variable is given, which is outside the limitations, the procedure proceeds calculation without the variable and sends out a warning.
;
;PARAMETERS
;5x7 array of distribution parameters:
;columns 1-3: parameters for the linear, power law or quadratic function which calculates the mean of the distribution
;columns 4-7: parameters to be added/subtracted from the geometric distribution to obtain adjusted distribution
;
;The output is a 6x4 array of probabilities (OUTPUT_DATA):
;Columns represent separate distributions:
;column 1: probability based on speed
;column 2: probability based on source
;column 3: probability based on width
;column 4: probability based on flare
;column 5: probability based on interaction
;column 6: combined probability
;Rows represent separate Dst bins (i.e. distribution bins k, k=1,2,3,4):
;row 1: < 100nT (k=1)
;row 2: 100-200nT (k=2)
;row 3: 200-300 nT (k=3)
;row 4: > 300 nT (k=4)
;
;*****************************************************************************************************************************************************************
;
;CALLS UPPON:
;linear.pro (function)
;power.pro (function)
;quadratic.pro (function)
;gd_probability.pro (function)
;make_adj_distribution.pro (procedure)
;
;*****************************************************************************************************************************************************************
;
;Function "linear" calculates linear function based on the input variable and 2 parameters:
FUNCTION linear, input, p_1, p_2
         RETURN, p_1*input+p_2
END

;Function "power" calculates power-law function based on the input variable and 2 parameters:
FUNCTION power, input, p_1, p_2
         RETURN, p_1*(input^p_2)
END

;Function "quadratic" calculates quadratic function based on the input variable and 3 parameters:
FUNCTION quadratic, input, p_1, p_2, p_3
         RETURN, p_1*input^2+p_2*input+p_3
END

;Function "gd_probability" calculates the probability of succes in each trial for the geometric probability distribution,
; using mean of the distribution as an input:
FUNCTION gd_probability, mean
         RETURN, 1/(1+mean/100)
END

;The procedure "make_adj_distribution" calculates adjusted distribution based on the geometric distribution (GD).
;Input:
;prob = scalar denoting the probability of succes in each trial for the GD.
;add = 1x4 array of values to be added to GD (1 value per each bin k, k=1,2,3,4) in order to obtain adjusted distribution
;Output:
;ADJ_out = 1x4 array of probabilities for k=1,2,3,4 (i.e. adjusted distribution)
;
PRO make_adj_distribution, prob, add, ADJ_out
    s=float(0)
    GD_out=make_array(1,4,/float)
    for k=0,3,1 do begin
          GD_out(k)=prob*(1-prob)^(k+1)
          s=s+GD_out(k)
    endfor
    for k=0,3,1 do begin
      ADJ_out(k)=GD_out(k)/s+add(k)
    endfor
END

;main
pro cgeftgdm_probability, input_data, params, output_data, output_text

;define variables
  
  output_text=make_array(7,1, /string)
  output_data=make_array(6,4, /float)
  output=make_array(1,4, /float)
  speed_add=params[0,3:6]
  source_add=params[1,3:6]
  width_add=params[2,3:6]
  flare_add=params[3,3:6]
  interaction_add=params[4,3:6]
  set=make_array(5,1, /string)

;MAKE DISTRIBUTIONS

  ;for speed
  if ((input_data(0) GE 400) and (input_data(0) LE 5000)) then begin 
      m=linear(input_data(0), params(0,0), params(0,1))
      p=gd_probability(m)
      make_adj_distribution, p, speed_add, output
      for k=0,3,1 do begin
          output_data(0,k)=output(k)
      endfor
      output_text(0,0)="Probability distribution for CME speed."
      set(0)='P_v,'
  endif else begin
      output_text(0,0)="No probability distribution for CME speed: input is missing."
      set(0)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(0,k)=output(k)
      endfor    
  endelse
   
      
  ;for source position
  if (input_data(1) GT 0) and (input_data(1) LE 1) then begin
      m=power(input_data(1), params(1,0), params(1,1))
      p=gd_probability(m)
      make_adj_distribution, p, source_add, output
      for k=0,3,1 do begin
          output_data(1,k)=output(k)
      endfor
      output_text(1,0)="Probability distribution for CME/flare source position radius."
      set(1)='P_r,'
  endif else begin
      output_text(1,0)="No probability distribution for CME/flare source position radius: input is missing."
      set(1)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(1,k)=output(k)
      endfor
  endelse
  
  ;for width
  if (input_data(2) EQ 1) or (input_data(2) EQ 2) or (input_data(2) EQ 3) then begin
      m=quadratic(input_data(2), params(2,0), params(2,1), params(2,2))
      p=gd_probability(m)
      make_adj_distribution, p, width_add, output
      for k=0,3,1 do begin
          output_data(2,k)=output(k)
      endfor
      output_text(2,0)="Probability distribution for CME width."
      set(2)='P_w,'
  endif else begin
      output_text(2,0)="No probability distribution for CME width: input is missing."
      set(2)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(2,k)=output(k)
      endfor
  endelse
  
  ;for flare
  if (input_data(3) EQ 1) or (input_data(3) EQ 2) or (input_data(3) EQ 3) then begin
      m=quadratic(input_data(3), params(3,0), params(3,1), params(3,2))
      p=gd_probability(m)
      make_adj_distribution, p, flare_add, output
      for k=0,3,1 do begin
          output_data(3,k)=output(k)
      endfor
      output_text(3,0)="Probability distribution for flare X-ray class."
      set(3)='P_f,'
  endif else begin
      output_text(3,0)="No probability distribution for flare X-ray class: input is missing."
      set(3)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(3,k)=output(k)
      endfor
  endelse
  
  ;for interaction level
  if (input_data(4) EQ 1) or (input_data(4) EQ 2) or (input_data(4) EQ 3) or (input_data(4) EQ 4) then begin
      m=power(input_data(4), params(4,0), params(4,1))
      p=gd_probability(m)
      make_adj_distribution, p, interaction_add, output
      for k=0,3,1 do begin
          output_data(4,k)=output(k)
      endfor
      output_text(4,0)="Probability distribution for interaction level."
      set(4)='P_i'
  endif else begin
      output_text(4,0)="No probability distribution for interaction level: input is missing."
      set(4)=''
      output=fltarr(4)
      for k=0,3,1 do begin
          output_data(4,k)=output(k)
      endfor
  endelse


  ;combined probability distribution
  s=float(0)
  output=output_data
  for n=0,3,1 do begin
      sum_0=float(0)
      sum_1=float(0)
      sum_2=float(0)
      sum_3=float(0)
      sum_4=float(0)
      for k=0,4,1 do begin
          sum_0=sum_0+output(k,n)
          for i=1,4,1 do begin
              if (i GT k) then begin
                  prod_1=output(k,n)*output(i,n)
                  sum_1=sum_1+prod_1
              endif
              for j=2,4,1 do begin
                  if (j GT i) AND (i GT k) then begin
                      prod_2=prod_1*output(j,n)
                      sum_2=sum_2+prod_2
                  endif
                  for l=3,4,1 do begin
                      if (l GT j) AND (j GT i) AND (i GT k) then begin
                          prod_3=prod_2*output(l,n)
                          sum_3=sum_3+prod_3
                      endif
                      for p=4,4,1 do begin
                          if (p GT l) AND (l GT j) AND (j GT i) AND (i GT k) then begin
                              prod_4=prod_3*output(p,n)
                              sum_4=sum_4+prod_4
                          endif
                      endfor;p
                  endfor;l
               endfor;j
           endfor;i
       endfor;k
       output(5,n)=sum_0-sum_1+sum_2-sum_3+sum_4
       s=s+output(5,n)
   endfor;n
   for k=0,3,1 do begin
          output_data(5,k)=output(5,k)/s
   endfor

output_text(5,0)='Combined probability distribution calculated based on ' + set(0)+set(1)+set(2)+set(3)+set(4)

end



