;DBM.pro
;
;Drag-based model for heliospheric propagation of coronal mass ejections (CMEs)
;
;INPUT FILE:
;
;dbm_input.txt
;=13x1 array of parameters:
;column 1: dd = day (mandatory)
;column 2: mm = month (mandatory)
;column 3: yyyy = year (mandatory)
;column 4: hh = hour (mandatory)
;column 5: min = minute (mandatory)
;column 6: R_0 = CME starting distance in Rsun (optional: default value 20)
;column 7: v_0 = CME starting speed in km/s (mandatory)
;column 8: gamma = drag parameter in 10^(-7) 1/km (optional: default value 0.2)
;column 9: w = solar wind speed in km/s (optional: default value 450)
;column 10: R_target = distance of the target in AU (optional: default value 1 - Earth)
;column 11: width = CME width in degrees (optional: if not set calculation proceedes with basic DBM)
;column 12: CMD = central meridian distance in degrees (optional: if not set calculation proceedes with basic DBM)
;column 13: sep_angle = target separation angle with respect to Earth (optional: if not set calculation proceedes with basic DBM)
;If mandatory values are missing DBM issues a warning.
;If optional values are missing (set to 0) DBM calculates with default values and/or calculation proceedes with basic DBM.
;
;
;OUTPUT FILE:
;
;dbm_output.txt
;
;*****************************************************************************************************************************************************************
;
;CALLS UPPON:
;
;*****************************************************************************************************************************************************************
;

PRO dbm

;define file path
path=file_dirname(routine_filepath("dbm"))

;read-in input data from file
openr,lun, path + '/INPUT/dbm_input.txt',/get_lun
dbm_input=make_array(13,1,/float)
readf, lun, dbm_input
free_lun,lun

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;conditions
output_text=make_array(4,1,/string)

;DATE & TIME CHECK
conditions=make_array(4,1)
conditions(0)=(dbm_input(1) LT 1) or (dbm_input(1) GT 12)
conditions(1)=(dbm_input(2) LT 1) or (dbm_input(2) GT 9999)
conditions(2)=(dbm_input(3) LT 0) or (dbm_input(3) GT 23)
conditions(3)=(dbm_input(4) LT 0) or (dbm_input(4) GT 59)
feb_check=dbm_input(2) MOD 4
if (dbm_input(1) EQ 2) then begin
  if (feb_check=0) then begin
    condition=(dbm_input(0) LT 1) or (dbm_input(0) GT 29)
  endif else begin
    condition=(dbm_input(0) LT 1) or (dbm_input(0) GT 28)
  endelse
endif else begin
  for i=1,7,2 do begin
    if (dbm_input(1) EQ i) then begin
      condition=(dbm_input(0) LT 1) or (dbm_input(0) GT 31)
      endif else begin
      condition=(dbm_input(0) LT 1) or (dbm_input(0) GT 30)
    endelse
  endfor
  for i=8,12,2 do begin
    if (dbm_input(1)EQ i) then begin
      condition=(dbm_input(0) LT 1) or (dbm_input(0) GT 31)
    endif else begin
      condition=(dbm_input(0) LT 1) or (dbm_input(0) GT 30)
    endelse
  endfor
endelse
for i=0,3,1 do begin
condition=condition or conditions(i)
endfor
if (condition) then begin
output_text(0)="CME liftoff date and time is wrong or missing. Calculation not possible."
;print out output text in a file
openw,lun, path + '/OUTPUT/dbm_output.txt',/get_lun
printf,lun,output_text
free_lun,lun
endif else begin

;BASIC DBM PARAMETER CHECK
if (dbm_input(6) LE 0) then begin
output_text(1)="CME speed is out of range or missing. Calculation not possible."
;print out output text in a file
openw,lun, path + '/OUTPUT/dbm_output.txt',/get_lun
printf,lun,output_text
free_lun,lun
endif else begin
if (dbm_input(5) LT 1) then (dbm_input(5)=20)
if (dbm_input(7) LT 0.0001) then (dbm_input(7)=0.2)
if (dbm_input(8) LT 1) then (dbm_input(8)=450)
if (dbm_input(9) LT 1) then (dbm_input(9)=1)

;INPUT VARIABLE DEFINITIONS
dd=long(dbm_input(0))
mm=long(dbm_input(1))
yyyy=long(dbm_input(2))
hh=long(dbm_input(3))
min=long(dbm_input(4))
date_in=strcompress(string(yyyy)+"/"+string(mm)+"/"+string(dd),/remove_all)+" "+strcompress(string(hh)+":"+string(min),/remove_all)
T_start=JULDAY(mm,dd,yyyy,hh,min,0)
R_0=dbm_input(5)*6.955E5
R_0in=string(long(dbm_input(5)))
v_0=dbm_input(6)
v_0in=string(long(v_0))
gamma=dbm_input(7)/10000000
gamma_in=string(float(dbm_input(7)))
w=dbm_input(8)
w_in=string(long(w))
R_target=dbm_input(9)
R_targetin=string(long(R_target))
width=dbm_input(10)
width_in=string(long(dbm_input(10)))
cmd=dbm_input(11)
cmd_in=string(long(dbm_input(11)))
sep_angle=dbm_input(12)
sep_anglein=string(long(dbm_input(12)))

;BASIC DBM
result=make_array(14400,4)
for i=0,14399,1 do begin
result(i,0)=i
if (v_0 LE w) then begin
result(i,1)=(-1/gamma)*alog(1-gamma*(v_0-w)*(result(i,0)*60))+w*(result(i,0)*60)+R_0
result(i,2)=(v_0-w)/(1-gamma*(v_0-w)*(result(i,0)*60))+w
endif else begin
result(i,1)=(1/gamma)*alog(1+gamma*(v_0-w)*(result(i,0)*60))+w*(result(i,0)*60)+R_0
result(i,2)=(v_0-w)/(1+gamma*(v_0-w)*(result(i,0)*60))+w
endelse
result(i,1)=result(i,1)/149597871
result(i,3)=Abs(result(i,1)-R_target)
endfor
min=min(result(0:14399,3),min_index)
TT=result(min_index,0)
r=result(min_index,1)
v=long(result(min_index,2))
v_out=strcompress(string(v)+"km/s")

;TRANSIT TIME
TT_hh=floor(TT/60)
TT_min=long((TT/60-TT_hh)*60)
TT_out=strcompress(string(TT_hh)+"h "+string(TT_min)+"min")

;ARRIVAL DATE & TIME
arr_dd=TT/(24*60)
T_arr=T_start+arr_dd
CALDAT,T_arr,mm_out,dd_out,yyyy_out,hh_out,min_out
arr_out=strcompress(string(yyyy_out)+"/"+string(mm_out)+"/"+string(dd_out),/remove_all)+" "+strcompress(string(hh_out)+":"+string(min_out),/remove_all)

;ADVANCED DBM PARAMETER CHECK
aconditions=make_array(2,1)
acondition=(dbm_input(10) GT 360) or (dbm_input(10) LT 1)
aconditions(0)=(dbm_input(11) GT 90) or (dbm_input(11) LT -90)
aconditions(1)=(dbm_input(12) GT 90) or (dbm_input(12) LT -90)
for i=0,1,1 do begin
acondition=acondition or aconditions(i)
endfor
if (acondition) then begin
output_text(2)="Parameters for advanced DBM are wrong or missing. Proceeding with basic DBM..."

;BASIC DBM OUTPUT
dbm_output=make_array(1,14,/string)
dbm_output(0,0)=output_text(2)
dbm_output(0,1)=" "
dbm_output(0,2)="DBM input:"
dbm_output(0,3)="CME take-off date & time: "+date_in
dbm_output(0,4)="CME take-off distance in Rsun: "+R_0in
dbm_output(0,5)="CME take-off speed in km/s: "+v_0in
dbm_output(0,6)="drag parameter gamma in 10E-7 1/km: "+gamma_in
dbm_output(0,7)="solar wind speed in km/s: "+w_in
dbm_output(0,8)="Target distance from the Sun in AU: "+R_targetin
dbm_output(0,9)=" "
dbm_output(0,10)="DBM output:"
dbm_output(0,11)="CME transit time: "+TT_out
dbm_output(0,12)="CME impact speed at target: "+v_out
dbm_output(0,13)="CME arrival time at target: "+arr_out

;print out output data in a file
openw,lun, path + '/OUTPUT/dbm_output.txt',/get_lun
printf,lun,dbm_output
free_lun,lun

endif else begin

;ADVANCED DBM
alpha=abs(sep_angle-cmd)
if (width LT 180) then begin
halfwidth=width/2
endif else begin
halfwidth=179./2
endelse

if (alpha GT halfwidth) then begin
output_text(3)="Under these conditions CME misses target."
;print out output text in a file
openw,lun, path + '/OUTPUT/dbm_output.txt',/get_lun
printf,lun,output_text
free_lun,lun
endif else begin

aresult=make_array(14400,6)
for i=0,14399,1 do begin
aresult(i,0)=result(i,0)
aresult(i,1)=result(i,1)
aresult(i,2)=result(i,2)
aresult(i,3)=result(i,1)*(cos(alpha)+sqrt((tan(halfwidth))^2-(sin(alpha))^2))/(1+tan(halfwidth))
aresult(i,4)=result(i,2)*(cos(alpha)+sqrt((tan(halfwidth))^2-(sin(alpha))^2))/(1+tan(halfwidth))
aresult(i,1)=aresult(i,1)/149597871
aresult(i,5)=Abs(aresult(i,3)-R_target)
endfor
min=min(aresult(0:14399,5),min_index)
TT=aresult(min_index,0)
r=aresult(min_index,3)
v=long(aresult(min_index,4))
v_out=strcompress(string(v)+"km/s")

;TRANSIT TIME
TT_hh=floor(TT/60)
TT_min=long((TT/60-TT_hh)*60)
TT_out=strcompress(string(TT_hh)+"h "+string(TT_min)+"min")

;ARRIVAL DATE & TIME
arr_dd=TT/(24*60)
T_arr=T_start+arr_dd
CALDAT,T_arr,mm_out,dd_out,yyyy_out,hh_out,min_out
arr_out=strcompress(string(yyyy_out)+"/"+string(mm_out)+"/"+string(dd_out),/remove_all)+" "+strcompress(string(hh_out)+":"+string(min_out),/remove_all)

;ADVANCED DBM OUTPUT
dbm_output=make_array(1,15,/string)
dbm_output(0,0)="DBM input:"
dbm_output(0,1)="CME take-off date & time: "+date_in
dbm_output(0,2)="CME take-off distance in Rsun: "+R_0in
dbm_output(0,3)="CME take-off speed in km/s: "+v_0in
dbm_output(0,4)="drag parameter gamma in 10E-7 1/km: "+gamma_in
dbm_output(0,5)="solar wind speed in km/s: "+w_in
dbm_output(0,6)="Target distance from the Sun in AU: "+R_targetin
dbm_output(0,7)="CME width in degrees: "+width_in
dbm_output(0,8)="CME source region central meridian distance: "+cmd_in
dbm_output(0,9)="Target separation angle from Earth in degrees: "+sep_anglein
dbm_output(0,10)=" "
dbm_output(0,11)="DBM output:"
dbm_output(0,12)="CME transit time: "+TT_out
dbm_output(0,13)="CME impact speed at target: "+v_out
dbm_output(0,14)="CME arrival time at target: "+arr_out

;print out output data in a file
openw,lun, path + '/OUTPUT/dbm_output.txt',/get_lun
printf,lun,dbm_output
free_lun,lun

endelse
endelse
endelse
endelse

end
