;FDFT.pro
;
;Forbush Decrease Forecast Tool (FDFT)
;based on the empirical statistical model for predicting Forbush decrease magnitude
;
;INPUT FILES:
;
;fdft.txt
;=6x1 array of parameters:
;column 1: v - speed of CME in km/s, float, range v > 106 km/s
;column 2: r - source position distance from the center of the solar disc in Rsun, float, range 0 ≤ r ≤ 1
;column 3: w - width of a CME, integer, range 0deg < w ≤ 360deg
;column 4: f - flare soft X-ray peak intensity in units 10−7Wm−2, float, range f > 5.3
;column 5: i - interaction level, integer, possible values: 1(none), 2 (low), 3 (medium), 4 (high)
;column 6: file_name - file name for the output files, string
;If a value for one or more of the parameters 1-5 is given which is outside of the possible range of values, the calculation proceeds without this/these variable(s).
;At least one of the input parameters (1-5) has to be within the possible range of values in order to proceed with the calculation, otherwise the warning is sent out.
;
;parameters.txt
;=5x2 array of distribution parameters:
;column 1: parameter a for the y=ax+b linear function which calculates the mean of the distribution
;column 2: parameter b for the y=ax+b linear function which calculates the mean of the distribution
;
;thresholds.txt
;=5x1 array of threshold parameters:
;column 1: T1 threshold for P(k=3)
;column 2: T2 threshold for P(k=2)
;column 3: T3 threshold for P(k=0)
;column 4: T4 threshold for P(k=1)
;column 5: T5 threshold for P(k=1)
;
;OUTPUT FILES:
;
;fdft_distributions.txt
;= 6x4 array of distributions:
;column 1: probability based on speed
;column 2: probability based on source
;column 3: probability based on width
;column 4: probability based on flare
;column 5: probability based on interaction
;column 6: combined probability
;Rows represent separate FD bins:
;row 1: less than 1%
;row 2: 1-3%
;row 3: 3-6%
;row 4: more than 6%
;
;fdft_table_text.txt
;= 4x6 array of text for each of the coulmn of fdft_distributions.txt
;
;fdft_text.txt
;= 1x2 array of online text:
;row 1: expected FD range
;row 2: corresponding image name
;
;fdft.png
;= image
;
;*****************************************************************************************************************************************************************
;
;CALLS UPPON:
;gdm_probability.pro
;FD_prediction.pro
;aarow.pro
;
;*****************************************************************************************************************************************************************
;

PRO fdft

;define file path
path=file_dirname(routine_filepath("fdft"))
pref_set, "IDL_PATH", path + ";<IDL_DEFAULT>", /COMMIT
resolve_all

;read-in input (solar) data from file
openr,lun, path + '\INPUT\fdft_input.txt',/get_lun
solar_data=make_array(5,1,/float)
readf, lun, solar_data
free_lun,lun
   
;read-in parameters from file
openr,lun,path + '\MODEL_DATA\fdft_parameters.txt',/get_lun
parameters=make_array(5,2,/float)
readf,lun,parameters
free_lun,lun

;read-in thresholds from file
openr,lun,path + '\MODEL_DATA\fdft_thresholds.txt',/get_lun
thresholds=make_array(5,1,/float)
readf,lun,thresholds
free_lun,lun

;make distributions (call fdftgdm_probability.pro)
fdftgdm_probability, solar_data, parameters, distributions, output_text

;make predictions (call FD_prediction.pro)
p_combined=make_array(1,4,/float)
p_combined=distributions(5,0:3)
fd_prediction, p_combined, thresholds, FD_expected

;plot predictions using Z-buffer:
thisDevice = !D.Name
Set_Plot, 'Z'
Erase
Device, Set_Resolution=[750,500],Set_Pixel_Depth=24, Decomposed=1

;find decomposed color:
;clr=[R,G,B], R,G,B=0,255
;somecolor = clr[0] + clr[1]*2L^8 + clr[2]*2L^16
;print, somecolor

labels=['< 1%','1-3%','3-6%','>6%']
c=11229501;cobalt;13107200;55295;gold;0;black;4605510;charcoal;13107200;blue;200 red
colors=[c,c,c,c]
c1=16777215;13499135;lemonchiffon;7346457;midnight blue;;12701133;ivory gray;4605510;charcoal;7237230;dark gray;13816530;medium gray;15132390;light gray;16777215;white
c2=0;55295;gold;7346457;midnight blue;16777215;16777215;white;0;black
c3=51200;green
c4=55295;gold
c5=42495;orange
c6=200;red
xtitle='!3FD (%)'
ytitle='!3Probability'
title='Probability distribution and expected FD range'
figure_name=path + '\OUTPUT\fdft_image.png'

;text_box:
text_box=make_array(1,6, /string)

text_box(0)='INPUT:'

if ((solar_data(0) GT 106) and (solar_data(0) LE 5000)) then begin
text_box(1)='v = '+ strtrim(string(solar_data(0),format='(f10.2)'),2)+' km/s'
endif else begin
text_box(1)='v = missing'
endelse

if ((solar_data(1) GE 0) and (solar_data(1) LE 1)) then begin
text_box(2)='r = '+strtrim(string(solar_data(1),format='(f10.2)'),2)+' Rsun'
endif else begin
text_box(2)='r = missing'
endelse

if ((solar_data(2) GT 0) and (solar_data(2) LE 360)) then begin
text_box(3)='w = '+strtrim(string(solar_data(2),format='(f10.2)'),2)+' deg'
endif else begin
text_box(3)='w = missing'
endelse

if (solar_data(3) GT 5.3) then begin
text_box(4)='f = '+strtrim(string(solar_data(3),format='(f10.2)'),2)+' 10^(-7)Wm^(-2)'
endif else begin
text_box(4)='f = missing'
endelse

case solar_data(4) of
1: text_box(5)='i = no interaction'
2: text_box(5)='i = interaction not likely'
3: text_box(5)='i = probable interaction'
4: text_box(5)='i = interaction highly probable'
else: text_box(5)='i = missing'
endcase

xl=2
xloc=[xl,xl,xl,xl,xl,xl]
yloc=[0.9,0.83,0.76,0.69,0.62,0.55]

;Figures:
case FD_expected of
0: begin
      output_text(6,0)='FD < 1%'
          
      !Y.RANGE = [0,1]
      !X.RANGE = [0,4]
      !P.CHARSIZE = 1.5
      !P.CHARTHICK = 1
      !P.FONT = 2
      !X.MARGIN = [8,2]
      !Y.MARGIN = [4,3]
      !P.COLOR = c2
      Bar_Plot, p_combined, barnames=labels, barspace=0.02, colors=colors, title=title, xtitle=xtitle, ytitle=ytitle, /outline, background=c1
      aarrow, 0.5, 0.87, 0.5, 0.72, /Data, fill=c3, length=5, width=20, shaft=10, color=c3
      XYOuts, xloc, yloc, text_box, /data, charsize=1.3, charthick=1, color=c2
      image = tvrd(true=1)
      write_png, figure_name, image
     end
    
1: begin
      output_text(6,0)='1% < FD < 3%'

      !Y.RANGE = [0,1]
      !X.RANGE = [0,4]
      !P.CHARSIZE = 1.5
      !P.CHARTHICK = 1
      !P.FONT = 2
      !X.MARGIN = [8,2]
      !Y.MARGIN = [4,3]
      !P.COLOR = c2
      Bar_Plot, p_combined, barnames=labels, barspace=0.02, colors=colors, title=title, xtitle=xtitle, ytitle=ytitle, /outline, background=c1
      aarrow, 1.5, 0.55, 1.5, 0.40, /Data, fill=c4, length=5, width=20, shaft=10, color=c4
      XYOuts, xloc, yloc, text_box, /data, charsize=1.3, charthick=1, color=c2
      image = tvrd(true=1)
      write_png, figure_name, image
     end
    
2: begin
      output_text(6,0)='3% < FD < 6%'

      !Y.RANGE = [0,1]
      !X.RANGE = [0,4]
      !P.CHARSIZE = 1.5
      !P.CHARTHICK = 1
      !P.FONT = 2
      !X.MARGIN = [8,2]
      !Y.MARGIN = [4,3]
      !P.COLOR = c2
      Bar_Plot, p_combined, barnames=labels, barspace=0.02, colors=colors, title=title, xtitle=xtitle, ytitle=ytitle, /outline, background=c1
      aarrow, 2.5, 0.45, 2.5, 0.30, /Data, fill=c5, length=5, width=20, shaft=10, color=c5
      XYOuts, xloc, yloc, text_box, /data, charsize=1.3, charthick=1, color=c2
      image = tvrd(true=1)
      write_png, figure_name, image
     end
     
3: begin
      output_text(6,0)='FD > 6%'

      !Y.RANGE = [0,1]
      !X.RANGE = [0,4]
      !P.CHARSIZE = 1.5
      !P.CHARTHICK = 1
      !P.FONT = 2
      !X.MARGIN = [8,2]
      !Y.MARGIN = [4,3]
      !P.COLOR = c2
      Bar_Plot, p_combined, barnames=labels, barspace=0.02, colors=colors, title=title, xtitle=xtitle, ytitle=ytitle, /outline, background=c1
      aarrow, 3.5, 0.40, 3.5, 0.25, /Data, fill=c6, length=5, width=20, shaft=10, color=c6
      XYOuts, xloc, yloc, text_box, /data, charsize=1.3, charthick=1, color=c2
      image = tvrd(true=1)
      write_png, figure_name, image
     end
endcase

Set_Plot, thisDevice  

out_array=strarr(1,4)
out_array(0)='FD < 1%'
out_array(1)='1% < FD < 3%'
out_array(2)='3% < FD < 6%'
out_array(3)='FD > 6%'
;str={text:strarr(1,4),data:fltarr(6,4)}
;str.text=out_array
;str.data=distributions


;save distributions in a text file:
openw,lun,path + '\OUTPUT\fdft_distributions.txt',/get_lun
printf,lun,format='("FD_range",5x,"P_v",5x,"P_r",5x,"P_w",5x,"P_f",5x,"P_i",5x,"P_combined")'
printf,lun,distributions(0:5,0),format='("FD < 1%",1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4)'
printf,lun,distributions(0:5,1),format='("1% < FD < 3%",1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4)'
printf,lun,distributions(0:5,2),format='("3% < FD < 6%",1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4)'
printf,lun,distributions(0:5,3),format='("FD > 6%",1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4,1x,f10.4)'
free_lun,lun

table_text=make_array(6,1, /string)
table_text=output_text(0:5,0)
online_text=make_array(1,4, /string)
online_text(0,0)='Based on the probability distribution for a given CME (blue histogram), the expected FD range calculated by the model is '$
 + output_text(6,0) + ' (marked by arrow).'
online_text(0,1)=output_text(5,0) + '(see table and table description). '
online_text(0,2)='For model calculation details see documentation.'
online_text(0,3)=figure_name

;save table_text in a text file:
openw,lun,path + '\OUTPUT\fdft_table_text.txt',/get_lun
printf,lun,table_text
free_lun,lun

;save online_text in a text file:
openw,lun,path + '\OUTPUT\fdft_text.txt',/get_lun
printf,lun,online_text
free_lun,lun


end